Package: BTYD
Type: Package
Title: Implementing BTYD Models with the Log Sum Exp Patch
Version: 2.4.3
Authors@R: c(person("Lukasz", "Dziurzynski", role="aut"),
             person("Edward", "Wadsworth", role="aut"), 
             person("Peter", "Fader", role="ctb"), 
             person("Elea", "McDonnell Feit", role="ctb"), 
             person("Daniel", "McCarthy", role=c("aut", "ctb")),
             person("Bruce", "Hardie", role="ctb"),
             person("Arun", "Gopalakrishnan", role="ctb"), 
             person("Eric", "Schwartz", role="ctb"), 
             person("Yao", "Zhang", role="ctb"), 
             person("Gabi", "Huiber", role=c("ctb", "cre"), email="ghuiber@gmail.com"))
Maintainer: Gabi Huiber <ghuiber@gmail.com>
BugReports: https://github.com/ghuiber/BTYD/issues
Description: Functions for data preparation, parameter estimation, scoring, and plotting for the 
    BG/BB (Fader, Hardie, and Shang 2010 <doi:10.1287/mksc.1100.0580>), 
    BG/NBD (Fader, Hardie, and Lee 2005 <doi:10.1287/mksc.1040.0098>) and 
    Pareto/NBD and Gamma/Gamma (Fader, Hardie, and Lee 2005 <doi:10.1509/jmkr.2005.42.4.415>) models.
License: GPL-3
Collate: 'data.R' 'BTYD.R' 'bgbb.R' 'bgnbd.R' 'pnbd.R' 'dc.R' 'spend.R'
Depends: R(>= 3.5), hypergeo, optimx, dplyr
Imports: Matrix
Suggests: knitr, rmarkdown
VignetteBuilder: knitr, rmarkdown
Author: Lukasz Dziurzynski [aut],
  Edward Wadsworth [aut],
  Peter Fader [ctb],
  Elea McDonnell Feit [ctb],
  Daniel McCarthy [aut, ctb],
  Bruce Hardie [ctb],
  Arun Gopalakrishnan [ctb],
  Eric Schwartz [ctb],
  Yao Zhang [ctb],
  Gabi Huiber [ctb, cre]
NeedsCompilation: no
Encoding: UTF-8
RoxygenNote: 7.1.2
Packaged: 2021-11-17 19:01:20 UTC; ghuiber
Repository: CRAN
Date/Publication: 2021-11-17 19:30:02 UTC
Built: R 4.6.0; ; 2026-04-29 00:14:32 UTC; unix
